/* rpi.c - Provides routines specific to memu-pi */

#include "types.h"
#include "diag.h"
#include "config.h"
#include "common.h"
#include "gpio.h"

#include <unistd.h>
/* #include <linux/reboot.h> */
#include <sys/reboot.h>
#include <stdio.h>
#include <sys/ioctl.h>
#include <linux/kd.h>
#include <termios.h>

#define	JOY1_FIRE	 GPIO_PIN(8)
#define	JOY1_UP		 GPIO_PIN(11)
#define	JOY1_DOWN	 GPIO_PIN(7)
#define	JOY1_LEFT	 GPIO_PIN(25)
#define	JOY1_RIGHT	 GPIO_PIN(9)
#define	JOY1_ALL	 ( JOY1_FIRE | JOY1_UP | JOY1_DOWN | JOY1_LEFT | JOY1_RIGHT )

#define	JOY2_FIRE	 GPIO_PIN(24)
#define	JOY2_UP		 GPIO_PIN(23)
#define	JOY2_DOWN	 GPIO_PIN(10)
#define	JOY2_LEFT	 GPIO_PIN(22)
#define	JOY2_RIGHT	 GPIO_PIN(21)
#define	JOY2_ALL	 ( JOY2_FIRE | JOY2_UP | JOY2_DOWN | JOY2_LEFT | JOY2_RIGHT )

BOOLEAN	bOS	=  FALSE;	/* True if running under a full OS */

int memu_entry (int argc, char *argv[]);

/* A replacement for the normal exit () system call, which halts the system */

void rpi_exit (int iStatus)
   {
   int iExit;
   /* Restore the GPIO */

   gpio_pullnone (JOY1_ALL | JOY2_ALL);
   gpio_term ();

   /* Exit to OS if available */

   if ( bOS )  exit (iStatus);

   /* Sync the file system to ensure all pending writes have completed */

   printf ("Memu-Pi shutting down.\n");
   sync ();
   sleep (2);
   sync ();
   sleep (2);
   printf ("Filesystem sinced.\n");

   /* Give time to read error message */

   if ( iStatus != 0 )  sleep (10);

   /* Halt the system */

   // printf ("Shutting down\n");
/*   iExit = reboot (LINUX_REBOOT_MAGIC1, LINUX_REBOOT_MAGIC2, LINUX_REBOOT_CMD_POWER_OFF, NULL); */
   iExit = reboot (RB_POWER_OFF);
   perror ("reboot: ");

   /* Failed to power down - wait forever */

   while (TRUE) sleep (1000000);
   }

static void uk_kbd_map (void)
	{
	static int ttyfd  =  0;
	struct kbentry kbe;
	struct
		{
		int	code;
		int shift;
		int key;
		} uk_map[] = {
		{ 0x03, K_SHIFTTAB, '"' },
			{ 0x28,  K_NORMTAB, '\'' },
			{ 0x28, K_SHIFTTAB, '@' },
			{ 0x2b, K_NORMTAB, '#' },
			{ 0x2b, K_SHIFTTAB, '~' },
			{ 0x56, K_NORMTAB, '\\' },
			{ 0x56, K_SHIFTTAB, '|' } };
	int i;

	for ( i = 0; i < sizeof (uk_map) / sizeof (uk_map[0]); ++i )
		{
		kbe.kb_table   =  uk_map[i].shift;
		kbe.kb_index   =  uk_map[i].code;
		kbe.kb_value   =  uk_map[i].key;
		if ( ioctl (ttyfd, KDSKBENT, &kbe) < 0 )
			diag_message (DIAG_HW_KBD, "Failed to map %s key 0x%02x to \'%c\'",
			( uk_map[i].shift == K_SHIFTTAB ? "Shifted" : "Normal" ), uk_map[i].code, uk_map[i].key);
		}
	}

int main (int argc, char *argv[])
	{
	int	i;
	/* Read configuration file */
	// printf ("argv[0] = %s\n", argv[0]);
	char *psCfg = cfg_find_file (argv[0]);
    // printf ("psCfg = %s\n", psCfg);
	if ( psCfg == NULL )
		{
		// printf ("Trying default configuration file.\n");
		psCfg = estrdup ("/memu/memu-pi.cfg");
		if ( ! cfg_test_file (psCfg) ) fatal ("No configuration file found");
		}
	if ( ! read_config (psCfg, &argc, &argv) ) fatal ("Error reading configuration file");
	printf ("Read configuration file.\n");
	free (psCfg);

	/* Test for OS switch */

	for ( i = 0; i < argc; ++i )	if ( ! strcmp (argv[i], "-os") )	bOS	  =	 TRUE;
	printf ("bOS = %d\n", bOS);

	if ( ! bOS )
		{
		/* Configure UK keyboard */
		uk_kbd_map ();
		/* Wait for end of kernel messages and then start MEMU */
		sleep (2);
		}

	/* Configure GPIO to read joysticks */

	if ( gpio_init () != GPIO_OK )	fatal ("Failed to initialise GPIO.");
	printf ("Initialised GPIO.\n");
	gpio_input (JOY1_ALL | JOY2_ALL);
	printf ("Set GPIO Inputs.\n");
	gpio_pullup (JOY1_ALL | JOY2_ALL);
	printf ("Set GPIO pullups.\n");

	printf ("Calling memu_entry.\n");
	memu_entry (argc, argv);
	}

byte joystick_sense1 (word drive)
	{
	byte sense =  0xff;
	int	 pins  =  gpio_get (JOY1_ALL | JOY2_ALL);
	if ( ( drive & 0x04 ) == 0 )
		{
		if ( ( pins & JOY1_UP ) == 0 )	  { sense &=	0x7f;  /* printf ("J1 up.\n"); */ }
		}
	if ( ( drive & 0x08 ) == 0 )
		{
		if ( ( pins & JOY1_LEFT ) == 0 )  { sense &=	0x7f;  /* printf ("J1 left.\n"); */ }
		}
	if ( ( drive & 0x10 ) == 0 )
		{
		if ( ( pins & JOY1_RIGHT ) == 0 ) { sense &=	0x7f;  /* printf ("J1 right.\n"); */ }
		}
	if ( ( drive & 0x20 ) == 0 )
		{
		if ( ( pins & JOY1_FIRE ) == 0 )  { sense &=	0x7f;  /* printf ("J1 fire.\n"); */ }
		}
	if ( ( drive & 0x40 ) == 0 )
		{
		if ( ( pins & JOY1_DOWN ) == 0 )  { sense &=	0x7f;  /* printf ("J1 down.\n"); */ }
		}
	if ( ( drive & 0x80 ) == 0 )
		{
		if ( ( pins & JOY2_LEFT ) == 0 )  { sense &=	0xfe;  /* printf ("J2 left.\n"); */ }
		if ( ( pins & JOY2_RIGHT ) == 0 ) { sense &=	0xfd;  /* printf ("J2 right.\n"); */ }
		if ( ( pins & JOY2_UP ) == 0 )	  { sense &=	0xfb;  /* printf ("J2 up.\n"); */ }
		if ( ( pins & JOY2_DOWN ) == 0 )  { sense &=	0xf7;  /* printf ("J2 down.\n"); */ }
		}
	return	sense;
	}

byte joystick_sense2 (word drive)
	{
	byte sense =  0xff;
	int	 pins  =  gpio_get (JOY1_ALL | JOY2_ALL);
	if ( ( drive & 0x80 ) == 0 )
		{
		if ( ( pins & JOY2_FIRE ) == 0 )  { sense &=	0xfe;  /* printf ("J2 fire.\n"); */ }
		}
	return	sense;
	}
